"use client";
import React, { useEffect, useMemo, useState } from "react";
import { Strip, StripEvent } from "./component/Strip";
import {
  Button,
  Container,
  createTheme,
  CssBaseline,
  Divider,
  Slide,
  Snackbar,
  Stack,
  ThemeProvider,
  Typography,
  useMediaQuery,
} from "@mui/material";
import { Bus } from "./component/Bus";
import Grid from "@mui/material/Unstable_Grid2";
import { EventCounter } from "@/utils/EventCounter";
import { BusesList } from "./component/BusesList";
import { isItMobile } from "@/utils/DetectMobile";

function random(min: number, max: number, floor: boolean = true) {
  const value = Math.random() * (max - min + 1) + min;
  return floor ? Math.floor(value) : value;
}

export default function Home() {
  const theme = useMemo(
    () =>
      createTheme({
        palette: { mode: "dark", background: { default: "#000" } },
      }),
    []
  );
  function onStripEvent(event: StripEvent) {
    console.log(event);
  }
  const isSnackBarHidden = useMemo(() => {
    if (typeof localStorage === "undefined") return false;
    return localStorage.getItem("snackBarHidden") === "true";
  }, []);
  const [snackBarVisible, setSnackBarVisibility] = useState(false);
  const breakPoints = useMemo(() => theme.breakpoints.values, []);
  const [width, setWidth] = useState(1000);
  const eventCounter = useMemo(() => {
    const eventCounter = new EventCounter<"emptySpaceClick">();
    eventCounter.on({
      callback: () => {
        if (document.fullscreenElement === null)
          document.documentElement.requestFullscreen();
        else document.exitFullscreen();
      },
      count: 3,
      name: "emptySpaceClick",
    });
    return eventCounter;
  }, []);
  const isItMobileDevice = useMemo(() => isItMobile(navigator.userAgent), []);
  useEffect(() => {
    setSnackBarVisibility(!isSnackBarHidden);
    setWidth(window.innerWidth);
  }, []);
  // useEffect(() => {
  //   const values = {} as Record<
  //     string,
  //     [number, "bigger" | "smaller" | "equal"]
  //   >;
  //   type valuesKey = keyof typeof breakPoints;
  //   for (const key in breakPoints) {
  //     if (Object.prototype.hasOwnProperty.call(breakPoints, key)) {
  //       const breakPointWidth = breakPoints[key as keyof typeof breakPoints];
  //       values[key as valuesKey] = [
  //         breakPointWidth,
  //         breakPointWidth === width
  //           ? "equal"
  //           : breakPointWidth > width
  //           ? "bigger"
  //           : "smaller",
  //       ];
  //     }
  //   }
  //   console.clear();
  //   console.log(
  //     width,
  //     `Closest: ${
  //       Object.entries(breakPoints).reduce((acc, i) => {
  //         const equation =
  //           Math.abs(i[1] - width) <= Math.abs(acc[1] - width) ? i : acc;
  //         return equation;
  //       })[0]
  //     }`
  //   );
  //   console.table(values);
  // }, [width]);
  const strips = { virtual: 3, physical: 5 };
  const buses = { virtual: 3, physical: 5 };
  const amountOfStrips = strips.physical + strips.virtual;
  const amountOfBuses = buses.physical + buses.virtual;
  const virtualInputNames = useMemo(() => ["VAIO", "AUX", "VAIO3"], []);
  if (typeof document !== "undefined") {
    document.onclick = () => eventCounter.emit("emptySpaceClick");
    window.onresize = () => {
      setWidth(window.innerWidth);
    };
  }

  const snackBarActionButton = (
    <>
      <Button
        onClick={() => {
          localStorage.setItem("snackBarHidden", "true");
          setSnackBarVisibility(false);
        }}
        color="secondary"
      >
        Gotcha
      </Button>
    </>
  );
  return (
    <ThemeProvider theme={theme}>
      <Container
        id="container"
        maxWidth={"xl"}
        style={{ height: "100%", paddingInline: "10px" }}
      >
        <div>
          <CssBaseline />
          <Typography variant="h5">Inputs</Typography>
          <Grid
            container
            // spacing={width ? 0 : 1}
            sx={{ paddingBlockStart: "15px" }}
            display={"flex"}
            alignItems={"center"}
            justifyContent={"space-evenly"}
            columns={{ xs: 8, sm: 12, md: 12, lg: 14 }}
          >
            {[...Array(amountOfStrips)].map((_v, stripId) => {
              const isPhysical = stripId < strips.physical;
              const name = `${isPhysical ? "A" : "B"}${
                isPhysical ? stripId + 1 : stripId + 1 - strips.physical
              }`;
              return (
                <React.Fragment key={name}>
                  <Grid
                    lgOffset={stripId === strips.physical ? 2 : 0}
                    mdOffset={
                      stripId === strips.physical ? (width > 1098 ? 0.8 : 0) : 0
                    }
                    sx={{ minWidth: "fit-content" }}
                  >
                    {/* <Typography variant="overline">{`Strip #${
                  stripId + 1
                }`}</Typography> */}
                    <Strip
                      onChange={(ev) => onStripEvent(ev)}
                      physicalBuses={buses.physical}
                      virtualBuses={buses.virtual}
                      width={width}
                      name={
                        isPhysical
                          ? `#${stripId + 1}`
                          : virtualInputNames[stripId - strips.physical]
                      }
                    />
                  </Grid>
                </React.Fragment>
              );
            })}
          </Grid>
          <Typography variant="h5">Outputs</Typography>
          <BusesList
            stackProps={{ sx: { paddingBlockStart: "10px" } }}
            width={width}
            physical={buses.physical}
            virtual={buses.virtual}
          />
        </div>
      </Container>
      <Snackbar
        anchorOrigin={{ horizontal: "center", vertical: "top" }}
        TransitionComponent={Slide}
        transitionDuration={250}
        open={snackBarVisible}
        message={
          <>
            <Typography fontWeight={600}>Tips:</Typography>
            <Typography component={"span"}>
              <Typography variant={"button"} fontWeight={500}>
                Double {isItMobileDevice ? "tap" : "click"}
              </Typography>{" "}
              on any gain slider to reset it's value to 0 dB
              <br />
            </Typography>
            <Typography component={"span"}>
              <Typography variant={"button"} fontWeight={500}>
                Triple {isItMobileDevice ? "tap" : "click"}
              </Typography>{" "}
              on empty space for toggle fullscreen
            </Typography>
          </>
        }
        action={snackBarActionButton}
      />
    </ThemeProvider>
  );
}
